import pytest

from jedi import settings
from jedi.evaluate.compiled import CompiledContextName


@pytest.fixture()
def auto_import_json(monkeypatch):
    monkeypatch.setattr(settings, 'auto_import_modules', ['json'])


def test_base_auto_import_modules(auto_import_json, Script):
    loads, = Script('import json; json.loads').goto_definitions()
    assert isinstance(loads._name, CompiledContextName)


def test_auto_import_modules_imports(auto_import_json, Script):
    main, = Script('from json import tool; tool.main').goto_definitions()
    assert isinstance(main._name, CompiledContextName)



"""
Test completions from *.pyc files:

 - generate a dummy python module
 - compile the dummy module to generate a *.pyc
 - delete the pure python dummy module
 - try jedi on the generated *.pyc
"""
import os
import shutil
import sys

import pytest

import jedi
from jedi.api.environment import SameEnvironment


SRC = """class Foo:
    pass

class Bar:
    pass
"""


@pytest.fixture
def pyc_project_path(tmpdir):
    path = tmpdir.strpath
    dummy_package_path = os.path.join(path, "dummy_package")
    os.mkdir(dummy_package_path)
    with open(os.path.join(dummy_package_path, "__init__.py"), 'w'):
        pass

    dummy_path = os.path.join(dummy_package_path, 'dummy.py')
    with open(dummy_path, 'w') as f:
        f.write(SRC)
    import compileall
    compileall.compile_file(dummy_path)
    os.remove(dummy_path)

    if sys.version_info.major == 3:
        # Python3 specific:
        # To import pyc modules, we must move them out of the __pycache__
        # directory and rename them to remove ".cpython-%s%d"
        # see: http://stackoverflow.com/questions/11648440/python-does-not-detect-pyc-files
        pycache = os.path.join(dummy_package_path, "__pycache__")
        for f in os.listdir(pycache):
            dst = f.replace('.cpython-%s%s' % sys.version_info[:2], "")
            dst = os.path.join(dummy_package_path, dst)
            shutil.copy(os.path.join(pycache, f), dst)
    try:
        yield path
    finally:
        shutil.rmtree(path)


def test_pyc(pyc_project_path):
    """
    The list of completion must be greater than 2.
    """
    path = os.path.join(pyc_project_path, 'blub.py')
    s = jedi.Script(
        "from dummy_package import dummy; dummy.",
        path=path,
        environment=SameEnvironment())
    assert len(s.completions()) >= 2
